@@ -33,10 +33,12 @@ func TestNotFrag(t *testing.T) {
 	}
 }
 
-func TestDefragPing1(t *testing.T) {
+func TestDefragPingMultipleFrags(t *testing.T) {
 	defrag := NewIPv4Defragmenter()
 
-	// We inject the 4 fragment and test the DefragIPv4 interface
+	// We inject the 4 fragments and test the DefragIPv4 interface
+	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
+	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
 	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
 	gentestDefrag(t, defrag, testPing1Frag3, false, "Ping1Frag3")
 	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
@@ -57,6 +59,43 @@ func TestDefragPing1(t *testing.T) {
 	}
 }
 
+func TestDefragPing1(t *testing.T) {
+	defrag := NewIPv4Defragmenter()
+
+	// We inject the 4 fragments and test the DefragIPv4 interface
+	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
+	gentestDefrag(t, defrag, testPing1Frag3, false, "Ping1Frag3")
+	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
+	ip := gentestDefrag(t, defrag, testPing1Frag4, true, "Ping1Frag4")
+
+	if len(ip.Payload) != 4508 {
+		t.Fatalf("defrag: expecting a packet of 4508 bytes, got %d", len(ip.Payload))
+	}
+
+	validPayload := append(testPing1Frag1[34:], testPing1Frag2[34:]...)
+	validPayload = append(validPayload, testPing1Frag3[34:]...)
+	validPayload = append(validPayload, testPing1Frag4[34:]...)
+
+	if bytes.Compare(validPayload, ip.Payload) != 0 {
+		fmt.Println(bytediff.BashOutput.String(
+			bytediff.Diff(validPayload, ip.Payload)))
+		t.Errorf("defrag: payload is not correctly defragmented")
+	}
+
+	// We redo the same test to handle duplication, and be sure
+	// that the internal list is correctly cleaned up.
+	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
+	gentestDefrag(t, defrag, testPing1Frag3, false, "Ping1Frag3")
+	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
+	ip2 := gentestDefrag(t, defrag, testPing1Frag4, true, "Ping1Frag4")
+
+	if bytes.Compare(ip2.Payload, ip.Payload) != 0 {
+		fmt.Println(bytediff.BashOutput.String(
+			bytediff.Diff(validPayload, ip.Payload)))
+		t.Errorf("defrag: ip and ip2 payload are different")
+	}
+}
+
 func TestDefragPing1and2(t *testing.T) {
 	debug = false
 	defrag := NewIPv4Defragmenter()
@@ -107,21 +146,30 @@ func TestDefragPing1and2(t *testing.T) {
 func TestDefragPingTooMuch(t *testing.T) {
 	defrag := NewIPv4Defragmenter()
 
-	// We inject the 7 fragments, and expect to hit an error at the
-	// 8th fragment
-	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
-	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
-	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
-	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
-	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
-	gentestDefrag(t, defrag, testPing1Frag2, false, "Ping1Frag2")
-	gentestDefrag(t, defrag, testPing1Frag1, false, "Ping1Frag1")
+	ip1 := layers.IPv4{
+		Version:    4,
+		TTL:        15,
+		SrcIP:      net.IPv4(1, 1, 1, 1),
+		DstIP:      net.IPv4(2, 2, 2, 2),
+		Id:         0xcc,
+		FragOffset: 0,
+		Length:     500,
+		Flags:      layers.IPv4MoreFragments,
+	}
+	defrag.DefragIPv4(&ip1)
+	for i := 2; i < 8; i = i + 1 {
+		nip := ip1
+		nip.FragOffset = ip1.Length * uint16(i)
+		out, err := defrag.DefragIPv4(&nip)
+		if err != nil || out != nil {
+			t.Fatalf("defrag: %s", err)
+		}
 
-	p := gopacket.NewPacket(testPing1Frag1, layers.LinkTypeEthernet,
-		gopacket.Default)
-	ip, _ := p.Layer(layers.LayerTypeIPv4).(*layers.IPv4)
+	}
+	ip8 := ip1
+	ip8.FragOffset = 666
 
-	_, err := defrag.DefragIPv4(ip)
+	_, err := defrag.DefragIPv4(&ip8)
 	if err == nil {
 		t.Fatalf("defrag: Maximum number of fragments are supposed to be 8")
 	}
