@@ -11,7 +11,10 @@ package layers
 import (
 	"encoding/binary"
 	"encoding/hex"
+	"net"
 	"testing"
+
+	"github.com/google/gopacket"
 )
 
 // Test the function getIPv4OptionSize when the ipv4 has no options
@@ -56,6 +59,45 @@ func TestGetIPOptLengthOptionEndOfList(t *testing.T) {
 	}
 }
 
+// Tests that the Options slice is properly reset before parsing new data
+func TestIPOptResetDuringDecoding(t *testing.T) {
+	ip := &IPv4{
+		Options: []IPv4Option{{OptionType: 42, OptionLength: 4, OptionData: make([]byte, 2)}},
+	}
+
+	ipWithoutOptions := &IPv4{
+		SrcIP:    net.IPv4(192, 168, 1, 1),
+		DstIP:    net.IPv4(192, 168, 1, 1),
+		Protocol: IPProtocolTCP,
+	}
+
+	ipBytes, err := serialize(ipWithoutOptions)
+
+	if err != nil {
+		t.Fatalf("Failed to serialize ip layer: %v", err)
+	}
+
+	err = ip.DecodeFromBytes(ipBytes, gopacket.NilDecodeFeedback)
+
+	if err != nil {
+		t.Fatalf("Failed to deserialize ip layer: %v", err)
+	}
+
+	if len(ip.Options) > 0 {
+		t.Fatalf("Options slice has stale data from previous packet")
+	}
+
+}
+
+func serialize(ip *IPv4) ([]byte, error) {
+	buffer := gopacket.NewSerializeBuffer()
+	err := ip.SerializeTo(buffer, gopacket.SerializeOptions{
+		FixLengths:       true,
+		ComputeChecksums: true,
+	})
+	return buffer.Bytes(), err
+}
+
 // Test the function checksum
 func TestChecksum(t *testing.T) {
 	testData := []struct {
